"""WAeUP data center.

The waeup data center cares for management of upload data and provides
tools for importing/exporting CSV data.
"""
import os
import sys
import grok
from datetime import datetime
from zope.interface import Interface
from waeup.interfaces import IWAeUPObject
from waeup.viewlets import Index, MainArea

class DataCenter(grok.Container):
    """A data center contains CSV files.
    """
    storage = os.path.join(os.path.dirname(__file__), 'files')
    
    def getFiles(self):
        """Get a list of files stored in `storage`.
        """
        result = []
        if not os.path.exists(self.storage):
            return result
        for filename in os.listdir(self.storage):
            fullpath = os.path.join(self.storage, filename)
            if not os.path.isfile(fullpath):
                continue
            result.append(DataCenterFile(fullpath))
        return result

    def setStoragePath(self, path, move=False):
        """Set the path where to store files.
        """
        path = os.path.abspath(path)
        self.storage = path

class IDataCenterFile(Interface):
    """A data center file.
    """
    pass


class DataCenterFile(object):
    """A description of a file stored in data center.
    """

    def __init__(self, context):
        self.context = context
        self.name = os.path.basename(self.context)
        self.size = self.getSize()
        self.uploaddate = self.getDate()

    def getDate(self):
        """Get a human readable datetime representation.
        """
        date = datetime.fromtimestamp(os.path.getctime(self.context))
        return date.strftime('%c')

    def getSize(self):
        """Get a human readable file size.
        """
        bytesize = os.path.getsize(self.context)
        size = "%s bytes" % bytesize
        units = ['kb', 'MB', 'GB']
        for power, unit in reversed(list(enumerate(units))):
            power += 1
            if bytesize >= 1024 ** power:
                size = "%.2f %s" % (bytesize/(1024.0**power), unit)
                break
        return size


class Content(grok.Viewlet):
    grok.viewletmanager(MainArea)
    grok.context(DataCenter)
    grok.view(Index)

class Upload(grok.View):
    grok.context(DataCenter)
    grok.name('upload')
    grok.template('master')

    def update(self, uploadfile=None, CANCEL=None, SUBMIT=None):
        if CANCEL is not None:
            self.redirect(self.url(self.context))
            return
        if not uploadfile:
            return
        try:
            filename = uploadfile.filename
            target = os.path.join(self.context.storage, filename)
            open(target, 'wb').write(uploadfile.read())
        except IOError:
            self.flash('Error while uploading file. Please retry.')
            self.flash('I/O error: %s' % sys.exc_info()[1])
            return
        self.redirect(self.url(self.context))
    
class UploadMain(grok.Viewlet):
    grok.viewletmanager(MainArea)
    grok.context(DataCenter)
    grok.view(Upload)
    
